@@ -78,7 +78,7 @@ pathlib = _LazyModule.load('pathlib', 'pathlib')
 platform = _LazyModule.load('platform', 'platform')
 functools = _LazyModule.load('functools', 'functools')
 subprocess = _LazyModule.load('subprocess', 'subprocess')
-__version__ = '0.1.0'
+__version__ = '0.2.1'
 
 
 @functools.lru_cache(1)
@@ -256,7 +256,7 @@ def _bash_quote_paths(paths, start, end):
         if end in s:
             s = s.replace(end, ''.join('\\%s' % i for i in end))
         out.add(start + s + end)
-    return out
+    return out, need_quotes
 
 
 BASH_COMPLETE_SCRIPT = r"""
@@ -347,7 +347,8 @@ def bash_completions(prefix, line, begidx, endidx, env=None, paths=None,
         from the environment and platform.
     quote_paths : callable, optional
         A functions that quotes file system paths. You shouldn't normally need
-        this as the default is acceptable 99+% of the time.
+        this as the default is acceptable 99+% of the time. This function should
+        a set of the new paths and a boolean for whether the paths were quoted.
 
     Returns
     -------
@@ -412,7 +413,8 @@ def bash_completions(prefix, line, begidx, endidx, env=None, paths=None,
         strip_len += 1
 
     if '-o noquote' not in complete_stmt:
-        out = quote_paths(out, '', '')
+        out, need_quotes = quote_paths(out, '', '')
+        strip_len += int(need_quotes)
     if '-o nospace' in complete_stmt:
         out = set([x.rstrip() for x in out])
 
@@ -1125,7 +1127,7 @@ def _quote_paths(paths, start, end, append_end=True):
             s = s.replace(end, ''.join('\\%s' % i for i in end))
         s = start + s + end if append_end else start + s
         out.add(s)
-    return out
+    return out, need_quotes
 
 
 def _joinpath(path):
@@ -1259,10 +1261,10 @@ def complete_path(prefix, line, start, end, ctx, cdpath=True, filtfunc=None):
     if cdpath:
         _add_cdpaths(paths, prefix)
     paths = set(filter(filtfunc, paths))
-    paths = _quote_paths({_normpath(s) for s in paths},
-                         path_str_start,
-                         path_str_end,
-                         append_end)
+    paths, _ = _quote_paths({_normpath(s) for s in paths},
+                            path_str_start,
+                            path_str_end,
+                            append_end)
     paths.update(filter(filtfunc, _dots(prefix)))
     paths.update(filter(filtfunc, _env(prefix)))
     return paths, lprefix
