@@ -157,8 +157,10 @@ class History:
         Parameters
         ----------
         cmd: dict
-            A dict contains informations of a command. It should contains
-            the following keys like ``inp``, ``rtn``, ``ts`` etc.
+            This dict contains information about the command that is to be
+            added to the history list. It should contain the keys ``inp``,
+            ``rtn`` and ``ts``. These key names mirror the same names defined
+            as instance variables in the ``HistoryEntry`` class.
         """
         pass
 
@@ -190,7 +192,7 @@ class History:
         Parameters
         ----------
         size: None or tuple of a int and a string
-            Detemines the size and units of what would be allowed to remain.
+            Determines the size and units of what would be allowed to remain.
         blocking: bool
             If set blocking, then wait until gc action finished.
         """
@@ -226,7 +228,9 @@ class DummyHistory(History):
 # -*- coding: utf-8 -*-
 """Implements JSON version of xonsh history backend."""
 os = _LazyModule.load('os', 'os')
+sys = _LazyModule.load('sys', 'sys')
 time = _LazyModule.load('time', 'time')
+json = _LazyModule.load('json', 'json')
 builtins = _LazyModule.load('builtins', 'builtins')
 # amalgamated collections
 threading = _LazyModule.load('threading', 'threading')
@@ -560,7 +564,10 @@ class JsonHistory(History):
         Parameters
         ----------
         cmd : dict
-            Command dictionary that should be added to the ordered history.
+            This dict contains information about the command that is to be
+            added to the history list. It should contain the keys ``inp``,
+            ``rtn`` and ``ts``. These key names mirror the same names defined
+            as instance variables in the ``HistoryEntry`` class.
 
         Returns
         -------
@@ -582,7 +589,7 @@ class JsonHistory(History):
         ----------
         at_exit : bool, optional
             Whether the JsonHistoryFlusher should act as a thread in the
-            background, or execute immeadiately and block.
+            background, or execute immediately and block.
 
         Returns
         -------
@@ -605,7 +612,7 @@ class JsonHistory(History):
         """
         Returns all history as found in XONSH_DATA_DIR.
 
-        yeild format: {'inp': cmd, 'rtn': 0, ...}
+        yield format: {'inp': cmd, 'rtn': 0, ...}
         """
         while self.gc and self.gc.is_alive():
             time.sleep(0.011)  # gc sleeps for 0.01 secs, sleep a beat longer
@@ -615,7 +622,14 @@ class JsonHistory(History):
             except ValueError:
                 # Invalid json file
                 continue
-            commands = json_file.load()['cmds']
+            try:
+                commands = json_file.load()['cmds']
+            except json.decoder.JSONDecodeError:
+                # file is corrupted somehow
+                if builtins.__xonsh_env__.get('XONSH_DEBUG') > 0:
+                    msg = 'xonsh history file {0!r} is not valid JSON'
+                    print(msg.format(f), file=sys.stderr)
+                continue
             for c in commands:
                 yield {'inp': c['inp'].rstrip(), 'ts': c['ts'][0]}
         # all items should also include session items
@@ -646,10 +660,10 @@ class JsonHistory(History):
 """Implements the xonsh history backend via sqlite3."""
 # amalgamated builtins
 # amalgamated collections
-json = _LazyModule.load('json', 'json')
+# amalgamated json
 # amalgamated os
 sqlite3 = _LazyModule.load('sqlite3', 'sqlite3')
-sys = _LazyModule.load('sys', 'sys')
+# amalgamated sys
 # amalgamated threading
 # amalgamated time
 # amalgamated xonsh.history.base
@@ -673,7 +687,7 @@ def _xh_sqlite_create_history_table(cursor):
     """Create Table for history items.
 
     Columns:
-        info - JSON formated, reserved for future extension.
+        info - JSON formatted, reserved for future extension.
     """
     cursor.execute("""
         CREATE TABLE IF NOT EXISTS xonsh_history
